local HL_Message_EventFrame = CreateFrame("Frame")
local HL_OnLoad_EventFrame = CreateFrame("Frame")

HL_Message_EventFrame:RegisterEvent("CHAT_MSG_CHANNEL")

HL_OnLoad_EventFrame:RegisterEvent("PLAYER_LOGIN")
HL_OnLoad_EventFrame:RegisterEvent("PLAYER_LOGOUT")

excludelist = {}
local HL_ChatFrame
HL_OnLoad_EventFrame:SetScript(
	"OnEvent",
	function(self, event)
		local frames = {
			ChatFrame1,
			ChatFrame2,
			ChatFrame3,
			ChatFrame4,
			ChatFrame5,
			ChatFrame6,
			ChatFrame7,
			ChatFrame8,
			ChatFrame9,
			ChatFrame10
		}
		if event == "PLAYER_LOGIN" then
			if HL_ChatIndex == nil then
				HL_ChatIndex = 1
			end
			HL_ChatFrame = frames[HL_ChatIndex]

			if h_sound_enabled == nil then
				h_sound_enabled = true
			end

			if h_notifications_enabled == nil then
				h_notifications_enabled = true
			end

			if phrases == nil then
				phrases = {}
			end
			DEFAULT_CHAT_FRAME:AddMessage("|cFFFF0000Highlighter v2.5.1 |rby Kro has successfully loaded. Type /hl help...")
		end
	end
)

HL_Message_EventFrame:SetScript(
	"OnEvent",
	function(self, event, ...)
		local arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9 = ...

		local msg = " " .. string.lower(arg1):gsub("{", " "):gsub("}", " ") .. " "

		local author = string.lower(arg2:hsplit("-")[1])
		local clearname = arg2:hsplit("-")[1]

		if Check_Match(msg) and Check_Exclude(author) then
			HL_ChatFrame:AddMessage(
				"Found match from " ..
					"|cFFFFFF00|Hplayer:" .. clearname .. "|h[" .. clearname .. "]|h|r" .. " from channel |cFF0070dd" .. arg9
			)
			HL_ChatFrame:AddMessage("|cff0070ddMessage: |r" .. arg1)
			HL_ChatFrame:AddMessage(
				"Actions: " ..
					"|cffff7eff|HHLCustomLink:whisper:" ..
						clearname ..
							"|h[Whisper]|h|r " ..
								"|cFFFFFF00|HHLCustomLink:invite:" ..
									clearname .. "|h[Invite]|h|r " .. "|cffff863b|HHLCustomLink:exclude:" .. clearname .. "|h[Exclude]|h|r "
			)
			if (h_sound_enabled == true) then
				PlaySound(12889, "master")
			end

			if (h_notifications_enabled == true) then
				FlashClientIcon()
			end
		end
	end
)

function Check_Exclude(author)
	for index, player in next, excludelist do
		if string.match(author, player) then
			return false
		end
	end
	return true
end

function Check_Match(msg)
	if not next(phrases) == nil then
		return false
	end
	local foundmatch = false
	for index, words in next, phrases do
		for index, word in next, words do
			if word:find("!") == 3 then
				if string.match(msg, word:gsub("!", "")) ~= nil then
					foundmatch = false
					break
				end
			elseif string.match(msg, word) == nil or string.find(word, "^!") then
				foundmatch = false
				break
			else
				foundmatch = true
			end
		end
		if foundmatch == true then
			return true
		end
	end
	return false
end

local function Highlight_SlashCommandHandler(msg)
	local _, _, cmd, args = string.find(msg, "%s?(%w+)%s?(.*)")

	if cmd == nil then
		cmd = "help"
	end

	local aliases = {
		["ls"] = "list",
		["rm"] = "remove"
	}
	for k, v in pairs(aliases) do
		cmd = cmd:gsub("^" .. k, v)
	end

	local commands = {
		["add"] = function(args)
			local words = {}
			local items = ""

			for ma in args:gmatch("|c.-item.-|r") do
				local id = ma:match("item:(%d+)")
				local itemname, itemlink = GetItemInfo(id)
				args:gsub(itemlink, "")
				items = items .. itemlink
				table.insert(words, string.lower(itemname))
			end

			args = args:gsub("|c.-item.-|r", "")

			for word in args:gmatch("%S+") do
				--REMINDER: %s fixes so that it only matches the start of a word since we added a space to the message during onevent
				-- COULD USE ^ instead
				table.insert(words, "%s" .. string.lower(word))
			end

			table.insert(phrases, words)
			DEFAULT_CHAT_FRAME:AddMessage('Added: "|cFFFF0000' .. args .. items .. '|r" to phrases.')
		end,
		["clear"] = function(args)
			phrases = {}
		end,
		["list"] = function(args)
			for index, words in next, phrases do
				local hlstring = index .. ". "
				for index, word in next, words do
					hlstring = hlstring .. word:gsub("%%s", "") .. " "
				end
				DEFAULT_CHAT_FRAME:AddMessage(hlstring)
			end
		end,
		["remove"] = function(args)
			table.remove(phrases, tonumber(args))
		end,
		["exclude"] = function(args)
			table.insert(excludelist, args)
			DEFAULT_CHAT_FRAME:AddMessage('Added "|cFFFF0000' .. args .. '|r" to exclude list')
		end,
		["clearexclude"] = function(args)
			excludelist = {}
		end,
		["excludelist"] = function(args)
			for index, player in next, excludelist do
				DEFAULT_CHAT_FRAME:AddMessage(index .. ": " .. player)
			end
		end,
		["togglesound"] = function(args)
			h_sound_enabled = not h_sound_enabled
			DEFAULT_CHAT_FRAME:AddMessage("Sound enabled:" .. tostring(h_sound_enabled))
		end,
		["output"] = function(args)
			local str = "Match output:"
			for index, frame in next, CHAT_FRAMES do
				local framename, _ = GetChatWindowInfo(index)
				if string.len(framename) == 0 then
					break
				end
				str = str .. "|cffff863b|HHLCustomLink:outputframe:" .. index .. "|h[" .. framename .. "]|h|r "
			end
			DEFAULT_CHAT_FRAME:AddMessage(str)
		end,
		["pause"] = function(args)
			HL_Message_EventFrame:UnregisterEvent("CHAT_MSG_CHANNEL")
		end,
		["resume"] = function(args)
			HL_Message_EventFrame:RegisterEvent("CHAT_MSG_CHANNEL")
		end,
		["help"] = function(args)
			DEFAULT_CHAT_FRAME:AddMessage(
				"|cFFFF0000/hl add <Phrase> |r-- Phrase containing words to highlight. Use suffix ! to exclude a word"
			)
			DEFAULT_CHAT_FRAME:AddMessage("|cFFFF0000/hl list|r -- Lists all phrases and their indices")
			DEFAULT_CHAT_FRAME:AddMessage("|cFFFF0000/hl ls |r-- Same as /hl list")
			DEFAULT_CHAT_FRAME:AddMessage("|cFFFF0000/hl remove <Index> |r-- Removes phrase with <Index> from list")
			DEFAULT_CHAT_FRAME:AddMessage("|cFFFF0000/hl rm <Index> |r-- Same as /hl remove <Index>")
			DEFAULT_CHAT_FRAME:AddMessage("|cFFFF0000/hl clear |r -- Removes all entries in phrase list")
			DEFAULT_CHAT_FRAME:AddMessage("|cFFFF0000/hl exclude <Player> |r-- Excludes messages from specific player")
			DEFAULT_CHAT_FRAME:AddMessage("|cFFFF0000/hl excludelist |r -- Lists all excluded players")
			DEFAULT_CHAT_FRAME:AddMessage("|cFFFF0000/hl clearexclude |r -- Removes all entries in exclude list")
			DEFAULT_CHAT_FRAME:AddMessage("|cFFFF0000/hl togglesound |r -- Toggles notification sound on/off")
			DEFAULT_CHAT_FRAME:AddMessage("|cFFFF0000/hl options |r -- Display options")
			DEFAULT_CHAT_FRAME:AddMessage("|cFFFF0000/hl output |r -- Change which chat tab to output matches")
			DEFAULT_CHAT_FRAME:AddMessage("|cFFFF0000/hl pause |r -- Pause matching")
			DEFAULT_CHAT_FRAME:AddMessage("|cFFFF0000/hl resume|r -- Resume matching")

			DEFAULT_CHAT_FRAME:AddMessage(
				'Example: "|cFFFF0000/hl add lf ony !lfg|r" will match "|cFFFF0000lf2m onyxia|r" but not "|cFFFF0000lfg onyxia|r"'
			)
		end,
		["options"] = function(args)
			local muted = h_sound_enabled and "Mute sound" or "Unmute sound"
			local notifications = h_notifications_enabled and "Disable Window Notifications" or "Enable Window Notifications"

			DEFAULT_CHAT_FRAME:AddMessage(
				"Options: |cffff863b|HHLCustomLink:togglesound:|h[" ..
					muted .. "]|h|r |cffff863b|HHLCustomLink:togglenotifications:|h[" .. notifications .. "]|h|r "
			)
		end,
		["default"] = function(args)
			DEFAULT_CHAT_FRAME:AddMessage("Unknown command try |cFFFF0000/hl help|r. ")
		end
	}

	if commands[cmd] == nil then
		commands["default"](args)
	else
		commands[cmd](args)
	end
end
SlashCmdList["HIGHLIGHT"] = Highlight_SlashCommandHandler
SLASH_HIGHLIGHT1 = "/hl"

function string:hsplit(inSplitPattern, outResults)
	if not outResults then
		outResults = {}
	end
	local theStart = 1
	local theSplitStart, theSplitEnd = string.find(self, inSplitPattern, theStart)
	while theSplitStart do
		table.insert(outResults, string.sub(self, theStart, theSplitStart - 1))
		theStart = theSplitEnd + 1
		theSplitStart, theSplitEnd = string.find(self, inSplitPattern, theStart)
	end
	table.insert(outResults, string.sub(self, theStart))
	return outResults
end

local OriginalSetHyperlink = ItemRefTooltip.SetHyperlink
function ItemRefTooltip:SetHyperlink(link, ...)
	if (link:find("^HLCustomLink")) then
		return
	end
	return OriginalSetHyperlink(self, link, ...)
end

hooksecurefunc(
	"ChatFrame_OnHyperlinkShow",
	function(...)
		local chatFrame, link, text, button = ...
		if type(text) == "string" and text:match("HLCustomLink:exclude") and not IsModifiedClick() then
			local player = text:match(":exclude:(.-)|")
			table.insert(excludelist, string.lower(player))

			DEFAULT_CHAT_FRAME:AddMessage(
				"Added " .. "|cFFFFFF00|Hplayer:" .. player .. "|h[" .. player .. "]|h|r" .. " to exclude list."
			)
		elseif type(text) == "string" and text:match("HLCustomLink:whisper") and not IsModifiedClick() then
			local player = text:match("whisper:(.-)|")
			ChatFrame_OpenChat("/w " .. player .. " ")
		elseif type(text) == "string" and text:match("HLCustomLink:invite") and not IsModifiedClick() then
			local player = text:match("invite:(.-)|")
			InviteUnit(player)
		elseif type(text) == "string" and text:match("HLCustomLink:togglesound") and not IsModifiedClick() then
			h_sound_enabled = not h_sound_enabled
			DEFAULT_CHAT_FRAME:AddMessage("Sound enabled: " .. tostring(h_sound_enabled))
		elseif type(text) == "string" and text:match("HLCustomLink:togglenotifications") and not IsModifiedClick() then
			h_notifications_enabled = not h_notifications_enabled
			DEFAULT_CHAT_FRAME:AddMessage("Notifications enabled: " .. tostring(h_notifications_enabled))
		elseif type(text) == "string" and text:match("HLCustomLink:outputframe") and not IsModifiedClick() then
			local frames = {
				ChatFrame1,
				ChatFrame2,
				ChatFrame3,
				ChatFrame4,
				ChatFrame5,
				ChatFrame6,
				ChatFrame7,
				ChatFrame8,
				ChatFrame9,
				ChatFrame10
			}
			HL_ChatIndex = tonumber(text:match("outputframe:(%d+)|"))
			HL_ChatFrame = frames[HL_ChatIndex]
			DEFAULT_CHAT_FRAME:AddMessage("Output Channel set to: " .. text:match("%[(%a+)%]"))
		end
	end
)

--test
